unit hiColorBox; { ColorBox ver 3.50}

interface

{$I share.inc}

uses Windows,Kol,Share,WinList,Messages;

const
   NONE     = 0;
   SCALE    = 1;
   MOSAIC   = 2;
   GRADIENT = 3;

FullColor = 47;

ColorValues: array [0..FullColor - 1] of TColor = (
  clBlack,clMaroon,clGreen,clOlive,clNavy,clPurple,clTeal,clGray,
  clSilver,clRed,clLime,clYellow,clBlue,clFuchsia,clAqua,clWhite,
  clMoneyGreen,clSkyBlue,clCream,clMedGray,clActiveBorder,clActiveCaption,
  clAppWorkSpace,clBackground,clBtnFace,clBtnHighlight,clBtnShadow,    
  clBtnText,clCaptionText,clDefault,clGrayText,clHighlight,clHighlightText,
  clInactiveBorder,clInactiveCaption,clInactiveCaptionText,clInfoBk,
  clInfoText,clMenu,clMenuText,clNone,clScrollBar,cl3DDkShadow,
  cl3DLight,clWindow,clWindowFrame,clWindowText );

ColorNames: array [0..FullColor - 1] of String = (
  'Black','Maroon','Green','Olive','Navy','Purple','Teal','Gray',
  'Silver','Red','Lime','Yellow','Blue','Fuchsia','Aqua','White',
  'MoneyGreen','SkyBlue','Cream','MedGray','ActiveBorder','ActiveCaption',
  'AppWorkSpace','Background','BtnFace','BtnHighlight','BtnShadow',    
  'BtnText','CaptionText','Default','GrayText','Highlight','HighlightText',
  'InactiveBorder','InactiveCaption','InactiveCaptionText','InfoBk',
  'InfoText','Menu','MenuText','None','ScrollBar','3DDkShadow',
  '3DLight','Window','WindowFrame','WindowText' );


type
 THIColorBox = class(THIWinList)
   private
     ColorRange:integer;
     ColorArr:PArray;
     ColorSize:integer;
     BackgroundImg:HBITMAP;
     procedure InitBackgroundImg;     
     procedure InitList;
     function  _val_arr_get(Var Item:TData; var Val:TData):boolean;
     function  _val_arr_count:integer;
     procedure _OnClick( Sender: PObj );
     procedure _OnResize( Sender: PObj ); override;
     function  _OnMeasureItem( Sender: PObj; Idx: Integer ):Integer;
     function  _OnDrawItem( Sender: PObj; DC: HDC; const Rect: TRect; ItemIdx: Integer;
                            DrawAction: TDrawAction; ItemState: TDrawState ): Boolean;
   public
     _prop_Colors:string;
     _prop_FormatColor:byte;
     _prop_FullColor:boolean;
     _prop_ReadOnly:byte;
     _prop_ColorSize:byte;
     _prop_DefColor:TColor;
     _prop_Gradient:boolean;
     _prop_Frame:boolean;     
     _prop_BumpText:boolean;
     _prop_DepthGradient:integer;
     _prop_DepthBack:integer;
     _prop_DarkColor:TColor;
     _prop_LightColor:TColor;
     _prop_DarkClrBack:TColor;
     _prop_LightClrBack:TColor;
     _prop_Horizontal:boolean;
     _prop_ColorFrame:TColor;
     _prop_ScaleImage:byte;
     _prop_InversGrad:boolean;
     _prop_InversBack:boolean;
     _prop_InversClrTxt:boolean;
     _prop_BackgroundImg:HBITMAP;
     _prop_IndentImg:integer;

     destructor Destroy; override;
     procedure Init; override;

     procedure _work_doFormatColor(var _Data:TData; Index:word);
     procedure _work_doFullColor(var _Data:TData; Index:word);
     procedure _work_doGradient(var _Data:TData; Index:word);
     procedure _work_doDepthGradient(var _Data:TData; Index:word);
     procedure _work_doFrame(var _Data:TData; Index:word);
     procedure _work_doDarkColor(var _Data:TData; Index:word);
     procedure _work_doLightColor(var _Data:TData; Index:word);
     procedure _work_doHorizontal(var _Data:TData; Index:word);
     procedure _work_doColorFrame(var _Data:TData; Index:word);
     procedure _work_doBumpText(var _Data:TData; Index:word);
     procedure _work_doBackgroundImg(var _Data:TData; Index:word);
     procedure _work_doInversClrTxt(var _Data:TData; Index:word);
     procedure _work_doInversGrad(var _Data:TData; Index:word);
     procedure _work_doInversBack(var _Data:TData; Index:word);
     procedure _work_doDarkClrBack(var _Data:TData; Index:word);
     procedure _work_doLightClrBack(var _Data:TData; Index:word);     
     procedure _work_doScaleImage(var _Data:TData; Index:word);
     procedure _work_doDepthBack(var _Data:TData; Index:word);
     procedure _work_doSelectColor(var _Data:TData; Index:word);
     procedure _var_ColorArray(var _Data:TData; Index:word);
     procedure _var_Index(var _Data:TData; Index:word);     
     procedure _var_CurrentColor(var _Data:TData; Index:word);
  end;

implementation

//-----------------------       ----------------------

type

  AGRBQuad = array [0..0] of RGBQuad;
  PAGRBQuad = ^AGRBQuad;

  PPoints = ^TPoints;
  TPoints = array[0..0] of TPoint;

  PColor = ^TColor;

function GetLightColor(Color: TColor; Light: Byte) : TColor;
var   fFrom: TRGB;
begin
  PColor(@fFrom)^:= Color2RGB(Color);
  Result := RGB(
    (FFrom.R*100 + (255 - FFrom.R) * Light) div 100,
    (FFrom.G*100 + (255 - FFrom.G) * Light) div 100,
    (FFrom.B*100 + (255 - FFrom.B) * Light) div 100
  );
end;

function RGB2HTML (Color: TColor): string;
var   fFrom: TRGB;
begin
  PColor(@fFrom)^:= Color2RGB(Color);
  Result:='#'+Int2Hex(FFrom.R, 2)+Int2Hex(FFrom.G, 2)+Int2Hex(FFrom.B, 2)
end;

function GetShadeColor(Color: TColor; Shade: Byte) : TColor;
var   fFrom: TRGB;
begin
  PColor(@fFrom)^:= Color2RGB(Color);
  Result := RGB(
    Max(0, FFrom.R - Shade),
    Max(0, FFrom.G - Shade),
    Max(0, FFrom.B - Shade)
  );
end;

procedure _Polyline(DC: HDC;const Points: array of TPoint);
begin
  Polyline(DC, PPoints(@Points)^, High(Points) + 1);
end;

procedure _Gradient(Sender:PControl; DC:HDC; cbRect:TRect; Gradient:boolean; DepthGradient:integer; LightColor,DarkColor,ColorFrame:TColor; Frame,Horizontal,InversGrad:boolean);
var   dr,dg,db:byte;
      ColorLight,ColorDark,lj,dj:TColor;
      t,j:integer;
      hdcMem:HDC;
      hdcBmp:HBITMAP;
      hWidth,hHeight,gWidth,gHeight:integer;
begin
   with Sender{$ifndef F_P}^{$endif} do begin
      Canvas.Brush.Color:= Color2RGB(ColorFrame);
      hWidth:= cbRect.Right - cbRect.Left;
      hHeight:= cbRect.Bottom - cbRect.Top;
      gWidth:= hWidth - 2;
      gHeight:= hHeight - 2;
      hdcMem:= CreateCompatibleDC(0);
      hdcBmp:= CreateCompatibleBitmap(DC,hWidth,hHeight);
      SelectObject(hdcMem, hdcBmp);
      if Frame then FillRect(hdcMem,MakeRect(0,0,hWidth,hHeight),Canvas.Brush.Handle);
      if Gradient then begin
         ColorLight:= GetLightColor(LightColor, max(min(DepthGradient,100),50));
         ColorDark:= GetLightColor(DarkColor, max(min(DepthGradient-20,100),1));
         lj := ToSysColor(ColorLight);
         dj := ToSysColor(ColorDark);
         dr := TRGB(lj).r - TRGB(dj).r;
         dg := TRGB(lj).g - TRGB(dj).g;
         db := TRGB(lj).b - TRGB(dj).b;
         if Frame and not Horizontal then
            for j:= 0 to gHeight - 1 do begin
               if InversGrad then
                  t:= (max(gHeight,1) - j)*100 div max(gHeight,1)
               else
                  t:= j*100 div max(gHeight,1);
               Canvas.Pen.Color := RGB(dr*t div 100 + TRGB(dj).r,dg*t div 100 + TRGB(dj).g,db*t div 100 + TRGB(dj).b);
               SelectObject(hdcMem,Canvas.Pen.Handle);
               _Polyline(hdcMem,[MakePoint(1,gHeight - j), MakePoint(hWidth - 1,gHeight - j)])
            end
         else if not Frame and not Horizontal then
            for j:= 0 to hHeight do begin
               if InversGrad then
                  t:= (max(hHeight,1) - j)*100 div max(hHeight,1)
               else
                  t:= j*100 div max(hHeight,1);
               Canvas.Pen.Color := RGB(dr*t div 100 + TRGB(dj).r,dg*t div 100 + TRGB(dj).g,db*t div 100 + TRGB(dj).b);
               SelectObject(hdcMem,Canvas.Pen.Handle);
               _Polyline(hdcMem,[MakePoint(0,hHeight - j), MakePoint(hWidth, hHeight - j)]);
            end
         else if Frame and Horizontal then
            for j:= 0 to gWidth - 1 do begin
               if InversGrad then
                  t:= (max(gWidth,1) - j)*100 div max(gWidth,1)
               else
                  t:= j*100 div max(gWidth,1);
               Canvas.Pen.Color := RGB(dr*t div 100 + TRGB(dj).r,dg*t div 100 + TRGB(dj).g,db*t div 100 + TRGB(dj).b);
               SelectObject(hdcMem,Canvas.Pen.Handle);
               _Polyline(hdcMem,[MakePoint(gWidth -  j,  1), MakePoint(gWidth - j, hHeight - 1)]);
            end
         else if not Frame and Horizontal then
            for j:= 0 to hWidth do begin
               if InversGrad then
                  t := (max(hWidth,1) - j)*100 div max(hWidth,1)
               else
                  t := j*100 div max(hWidth,1);
               Canvas.Pen.Color := RGB(dr*t div 100 + TRGB(dj).r,dg*t div 100 + TRGB(dj).g,db*t div 100 + TRGB(dj).b);
               SelectObject(hdcMem,Canvas.Pen.Handle);
               _Polyline(hdcMem,[MakePoint(hWidth - j, 0), MakePoint(hWidth - j, hHeight)]);
            end;
      end
      else begin
         Canvas.Brush.Color:= LightColor;
         if Frame then
            FillRect(hdcMem,MakeRect(1,1,hWidth-1,hHeight-1), Canvas.Brush.Handle)
         else
            FillRect(hdcMem,MakeRect(0,0,hWidth,hHeight), Canvas.Brush.Handle)
      end;
      BitBlt(DC,cbRect.Left,cbRect.Top,cbRect.Right-cbRect.Left,cbRect.Bottom-cbRect.Top,hdcMem,0,0,SRCCOPY);
      DeleteDC(hdcMem);
      DeleteObject(hdcBmp);
   end;
end;

function ScaleImage(Sender:PControl; bitmap:HBITMAP; ImgSize:integer; mode:byte):HBITMAP;
var   nbmp,bmp,bmprect:PBitmap;
      sw,nw,ih:integer;
      i:integer;
begin
   with Sender{$ifndef F_P}^{$endif} do begin
      Result:= 0;
      case mode of
         NONE: exit;
         SCALE: exit;
         GRADIENT: exit;
         MOSAIC: begin
            ih:= max(ImgSize, Sender.Canvas.TextExtent('W').cy);
            bmprect:= NewBitmap(0,0);
            bmprect.handle:= CopyImage(bitmap,IMAGE_BITMAP,0,0,LR_CREATEDIBSECTION);
            nw:= max(Width*bmprect.height*100 div (bmprect.width*ih) + 200,1);
            sw:= (nw*bmprect.width*ih div bmprect.height) div 100;
            nbmp:= NewBitmap(sw,ih);
            bmp:= NewBitmap(bmprect.width*ih div bmprect.height,ih);   
            SetStretchBltMode(bmp.Canvas.Handle, HALFTONE);
            bmprect.StretchDraw(bmp.canvas.handle,bmp.BoundsRect);
            i:= 0;
            repeat
               BitBlt(nbmp.Canvas.Handle,i*bmp.width,0,sw,ih,bmp.canvas.handle,0,0,SRCCOPY);
               inc(i);
            until i = nw;
            Result:= CopyImage(nbmp.handle,IMAGE_BITMAP,0,0,LR_CREATEDIBSECTION);
            free_and_nil(bmprect);
            free_and_nil(nbmp);
            free_and_nil(bmp);
         end;
      end;      
   end;
end;

procedure THIColorBox._work_doFormatColor;begin _prop_FormatColor:= ToInteger(_Data);end;
procedure THIColorBox._work_doFullColor;begin _prop_FullColor:= ReadBool(_Data);InitList;end;
procedure THIColorBox._work_doGradient;begin _prop_Gradient:= ReadBool(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doInversClrTxt;begin _prop_InversClrTxt:= ReadBool(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doInversGrad;begin _prop_InversGrad:= ReadBool(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doFrame;begin _prop_Frame:= ReadBool(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doDarkColor;begin _prop_DarkColor:= ToInteger(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doLightColor;begin _prop_LightColor:= ToInteger(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doHorizontal;begin _prop_Horizontal:= ReadBool(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doColorFrame;begin _prop_ColorFrame:= ToInteger(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doBumpText;begin _prop_BumpText:= ReadBool(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doDarkClrBack;begin _prop_DarkClrBack:= ToInteger(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doLightClrBack;begin _prop_LightClrBack:= ToInteger(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doInversBack;begin _prop_InversBack:= ReadBool(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doDepthGradient;begin _prop_DepthGradient:= ToInteger(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doDepthBack;begin _prop_DepthBack:= ToInteger(_Data);Control.Invalidate;end;
procedure THIColorBox._work_doBackgroundImg;
begin
   if not _IsBitmap(_Data) then exit;
   _prop_BackgroundImg:= CopyImage(ToBitmap(_Data).handle,IMAGE_BITMAP,0,0,LR_CREATEDIBSECTION);
   InitBackgroundImg;
end;
procedure THIColorBox._work_doScaleImage;
begin
   _prop_ScaleImage:= ToIntIndex(_Data);
   InitBackgroundImg;
end;

procedure THIColorBox.InitBackgroundImg;
begin
   if (_prop_BackgroundImg = 0) then exit;
   DeleteObject(BackgroundImg);
   BackgroundImg:= ScaleImage(Control, _prop_BackgroundImg, _prop_ColorSize + _prop_IndentImg*2, _prop_ScaleImage);   
   Control.Invalidate;
end;

//------------------------       ----------------------

procedure THIColorBox.Init;
var   Flags:TComboOptions;
begin
   Flags := [coNoIntegralHeight, coReadOnly, coOwnerDrawFixed];
   ColorSize:= max(_prop_ColorSize + _prop_IndentImg*2,_prop_ColorSize);
   Control := NewComboBox(FParent,Flags);
   Control.OnSelChange := _OnClick;
   Control.OnMeasureItem:= _OnMeasureItem;
   Control.OnDrawItem:= _OnDrawItem;

inherited;
   InitList;
   InitBackgroundImg;
   _OnResize(Control);
end;

procedure THIColorBox.InitList;
var   i:integer;
      str:string;
      valid:boolean;
      CList:PStrList;
begin
   Control.Clear;
   if _prop_FullColor then ColorRange:= FullColor else ColorRange:= 16; 
   str:= '';
   for i:= 0 to ColorRange - 1 do str:= str + ColorNames[i] + #13#10;
   SetStrings(str);
   for i:= 0 to ColorRange - 1 do Control.ItemData[i]:= Color2RGB(ColorValues[i]);
   CList:= NewStrList;
   CList.text:= _prop_Colors;
   if CList.Count <> 0 then
      for i:=0 to CList.Count - 1 do begin
         str:= CList.Items[i] + '=';
         Add(gettok(str,'='));
         Control.ItemData[Control.Count - 1]:= str2int(gettok(str,'=')); 
      end; 
   CList.free; 
   valid:= false;
   Control.CurIndex := 0;
   for i:= 0 to ColorRange - 1 do 
      if Control.ItemData[i] = dword(Color2RGB(_prop_DefColor)) then begin
         valid:= true;
         break;
      end;
   if not valid then exit;
   Control.CurIndex := i;
   Control.Invalidate;
end;
      
destructor THIColorBox.Destroy;
begin
   if ColorArr <> nil then Dispose(ColorArr);
   DeleteObject(BackgroundImg);
   inherited Destroy;
end;

procedure THIColorBox._OnResize;
begin
   inherited;
   InitBackgroundImg;
end;

procedure THIColorBox._OnClick;
var   dr,dg,db:TData;
      fFrom: TRGB;
begin
   inherited;
   case _prop_DataType of
      0: _hi_OnEvent(_event_onClick, Control.CurIndex);
      1: begin
            case _prop_FormatColor of
               0: _hi_OnEvent(_event_onClick, integer(Control.ItemData[Control.CurIndex]));
               1: _hi_OnEvent(_event_onClick, RGB2HTML(Control.ItemData[Control.CurIndex]));
               2: begin
                     fFrom := TRGB(Control.ItemData[Control.CurIndex]);
                     dtInteger(dr, FFrom.R);
                     dtInteger(dg, FFrom.G);
                     dr.ldata:= @dg;
                     dtInteger(db, FFrom.B);
                     dg.ldata:= @db;
                     _hi_onEvent_(_event_onClick, dr);                
                  end;
            end;
         end;
   end;
end;

function THIColorBox._OnMeasureItem;
var   fControl: PControl;
begin
   fControl:= PControl(Sender);
   Result:= max(ColorSize, fControl.Canvas.TextExtent('W').cy);
end;

//---------------------       -----------------------

function THIColorBox._OnDrawItem;
const   SHIFT_PICTURE = 6;

var     fControl: PControl;
        ARect,BRect,cbRect: TRect;
        _Flags: dword;
        hdcMem: HDC;
        dx,dy:integer; 
begin
   Result:= False;
   fControl:= PControl(Sender);

   with fControl{$ifndef F_P}^{$endif} do begin
      if (Rect.Bottom - Rect.Top > Height) then exit;
         dx:= Rect.Right - Rect.Left;
         dy:= Rect.Bottom - Rect.Top;
         if (_prop_ScaleImage <> GRADIENT) and (_prop_BackgroundImg <> 0) then begin
            hdcMem:= CreateCompatibleDC(0);
            SelectObject(hdcMem, BackgroundImg);
            BitBlt(DC, 0, Rect.Top, dx, dy, hdcMem, 0, 0, SRCCOPY);
            DeleteDC(hdcMem);
         end
         else if (_prop_ScaleImage = GRADIENT) then
            _Gradient(fControl,DC,Rect,true,_prop_DepthBack,_prop_LightClrBack,
                      _prop_DarkClrBack,0,false,true,_prop_InversBack);         

         if (Itemidx = Count - 1) then begin
            cbRect:= Rect;
            cbRect.Top:= cbRect.Bottom;
            cbRect.Bottom:= cbRect.Bottom + dy;            
            if (_prop_ScaleImage <> GRADIENT) and (_prop_BackgroundImg <> 0) then begin
               hdcMem:= CreateCompatibleDC(0);
               SelectObject(hdcMem, BackgroundImg);
               BitBlt(DC, cbRect.Left, cbRect.Top, dx, dy, hdcMem, 0, 0, SRCCOPY);
               DeleteDC(hdcMem);               
            end
            else if (_prop_ScaleImage = GRADIENT) and not (odsComboboxEdit in ItemState) then begin
               _Gradient(fControl,DC,cbRect,true,_prop_DepthBack,_prop_LightClrBack,
                         _prop_DarkClrBack,0,false,true,_prop_InversBack); 
            end;
         end; 

      cbRect:= Rect;
      
      if (odsSelected in ItemState) then
         _Gradient(fControl,DC,cbRect,_prop_Gradient,_prop_DepthGradient,_prop_LightColor,_prop_DarkColor,
                   _prop_ColorFrame,_prop_Frame,_prop_Horizontal,_prop_InversGrad)
      else begin
         if ((_prop_BackgroundImg = 0) or (_prop_ScaleImage = NONE)) and (_prop_ScaleImage <> GRADIENT) then begin
            Canvas.Brush.Color:= Color;
            FillRect(DC,cbRect,Canvas.Brush.Handle);
         end;
      end;

      ARect:= Rect;
      ARect.Left:= ARect.Left + _prop_ColorSize*2 + Canvas.TextExtent('W').cx;
      if (odsComboboxEdit in ItemState) then ARect.Left:= ARect.Left - 3;
      BRect:= ARect; 
      inc(BRect.Left); inc(BRect.Top); inc(BRect.Right); inc(BRect.Bottom);
      SetTextColor(DC,Color2RGB(Font.Color));
      SetBkMode(DC, Font.Color);

      _Flags:=  DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX;  

      if _prop_BumpText then begin
         if (odsSelected in ItemState) and _prop_Gradient then begin
            if _prop_InversClrTxt then
               SetTextColor(DC,Color2RGB(clBackground))
            else
               SetTextColor(DC,Color2RGB(clHighlightText));
            DrawText(DC, PChar(Items[ItemIdx]), -1, BRect, _Flags);
            if _prop_InversClrTxt then
               SetTextColor(DC,Color2RGB(clHighlightText))
            else
               SetTextColor(DC,Color2RGB(Font.Color));
         end
         else if not (odsSelected in ItemState) and (_prop_ScaleImage = GRADIENT) then begin
            SetTextColor(DC,Color2RGB(clBackground));
            DrawText(DC, PChar(Items[ItemIdx]), -1, BRect, _Flags);
            SetTextColor(DC,Color2RGB(clHighlightText));
         end
         else
            SetTextColor(DC,Color2RGB(Font.Color));
      end
      else if (odsSelected in ItemState) then begin
            if _prop_InversClrTxt then
               SetTextColor(DC,Color2RGB(clHighlightText))
            else
               SetTextColor(DC,Color2RGB(Font.Color));
         end
         else
            SetTextColor(DC,Color2RGB(Font.Color));

      DrawText(DC, PChar(Items[ItemIdx]), -1, ARect, _Flags);
      Result:= True;

      with cbRect do begin
         Top:= Top + (Bottom - Top - _prop_ColorSize) div 2;
         Left:= SHIFT_PICTURE;
         Bottom:= Top + _prop_ColorSize; 
         Right:= Left + _prop_ColorSize*2;
      end;
      FillRect(DC,cbRect,CreateSolidBrush(clBlack));
      FillRect(DC,MakeRect(cbRect.Left+1,cbRect.Top+1,cbRect.Right-1,cbRect.Bottom-1),
               CreateSolidBrush(ItemData[ItemIdx]));
   end;
end;

//------------------         --------------------

procedure THIColorBox._var_ColorArray;
begin
  if ColorArr = nil then
    ColorArr := CreateArray(nil,_val_arr_get,_val_arr_count,nil);
  dtArray(_Data, ColorArr);
end;

function THIColorBox._val_arr_get;
var   ind:integer;
begin
  ind := ToIntIndex(Item);
  Result := (ind >=0 )and(ind < Control.Count);
  if Result then dtInteger(Val,Control.ItemData[ind]);
end;

function THIColorBox._val_arr_count;
begin
  Result := Control.Count;
end;

procedure THIColorBox._var_Index;
begin
   dtInteger(_Data, Control.CurIndex);
end;

procedure THIColorBox._var_CurrentColor;
var   dt,di:TData;
      d:PData;
      fFrom: TRGB;
begin
   case _prop_FormatColor of
      0: dtInteger(_Data, integer(Control.ItemData[Control.CurIndex]));
      1: dtString(_Data, RGB2HTML(Control.ItemData[Control.CurIndex]));
      2: begin
            fFrom := TRGB(Control.ItemData[Control.CurIndex]);
            dtNull(di);
            dtInteger(dt,FFrom.R);
            AddMTData(@di,@dt,d);
            dtInteger(dt,FFrom.G);
            AddMTData(@di,@dt,d);
            dtInteger(dt,FFrom.B);
            AddMTData(@di,@dt,d);
            CopyData(@_Data,@di);      
            if (d <> nil) and not _IsNULL(d^) then FreeData(d);
         end;
   end;
end;

procedure THIColorBox._work_doSelectColor;
var   Clr: TColor;
      i:integer;
      valid:boolean;
begin
   Clr:= ReadInteger(_Data, _data_Value, _prop_DefColor);
   valid:= false;
   for i:= 0 to ColorRange - 1 do 
      if Control.ItemData[i] = dword(Color2RGB(Clr)) then begin
         valid:= true;
         break;
      end;
   if not valid then exit;
   Control.CurIndex := i;
   Control.Invalidate;
end;

end.